<?php

namespace Schaffen\SerasaRemessa\Cnab600;

use Schaffen\SerasaRemessa\Utils\Record;
use Schaffen\SerasaRemessa\Utils\RecordColumn;
use Schaffen\SerasaRemessa\Utils\Util;

class Detail extends Record {
    /**
     * @var int $code Codigo do Registro, fixo 1
     */
    private $code = 1;

    /**
     * @var int $sequence Sequência do detalhe
     */
    private $sequence;

    /**
     * @var string $operation Código da operação a ser realizada, 'I' ou 'E'
     */
    private $operation;

    /**
     * @param string $cnpj_branch_digit Digito da filial no CNPJ da impresa
     */
    private $cnpj_branch_digit;

    /**
     * @var array Template para criação das colunas
     */
    protected $templateColumns = [        
        [
            'name' => 'code',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 1,
            'size' => 1
        ],
        [
            'name' => 'operation',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 2,
            'size' => 1
        ],
        [
            'name' => 'cnpj_branch_digit',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 3,
            'size' => 6
        ],
        [
            'name' => 'occurrence_date',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 9,
            'size' => 8
        ],
        [
            'name' => 'contract_end_date',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 17,
            'size' => 8
        ],
        [
            'name' => 'nature_code',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 25,
            'size' => 3
        ],
        [
            'name' => 'embratel_code',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 28,
            'size' => 4
        ],
        [
            'name' => 'person_type',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 32,
            'size' => 1
        ],
        [
            'name' => 'person_fisrt_document_type',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 33,
            'size' => 1
        ],
        [
            'name' => 'person_first_document',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 34,
            'size' => 15
        ],
        [
            'name' => 'discharge_reason',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 49,
            'size' => 2
        ],
        [
            'name' => 'person_second_document_type',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 51,
            'size' => 1
        ],
        [
            'name' => 'person_second_document',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 52,
            'size' => 15
        ],
        [
            'name' => 'person_second_document_uf',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 67,
            'size' => 2
        ],
        [
            'name' => 'guarantor_type',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 69,
            'size' => 1
        ],
        [
            'name' => 'guarantor_fisrt_document_type',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 70,
            'size' => 1
        ],
        [
            'name' => 'guarantor_first_document',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 71,
            'size' => 15
        ],
        [
            'name' => 'blank_1',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 86,
            'size' => 2
        ],
        [
            'name' => 'guarantor_second_document_type',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 88,
            'size' => 1
        ],
        [
            'name' => 'guarantor_second_document',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 89,
            'size' => 15
        ],
        [
            'name' => 'guarantor_second_document_uf',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 104,
            'size' => 2
        ],
        [
            'name' => 'person_name',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 106,
            'size' => 70
        ],
        [
            'name' => 'person_birthdate',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 176,
            'size' => 8
        ],
        [
            'name' => 'person_father_name',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 184,
            'size' => 70
        ],
        [
            'name' => 'person_mother_name',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 254,
            'size' => 70
        ],
        [
            'name' => 'person_full_address',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 324,
            'size' => 45
        ],
        [
            'name' => 'person_neighborhood',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 369,
            'size' => 20
        ],
        [
            'name' => 'person_city',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 389,
            'size' => 25
        ],
        [
            'name' => 'person_uf',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 414,
            'size' => 2
        ],
        [
            'name' => 'person_cep',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 416,
            'size' => 8
        ],
        [
            'name' => 'unpaid_value',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 424,
            'size' => 15
        ],
        [
            'name' => 'contract_number',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 439,
            'size' => 16
        ],
        [
            'name' => 'our_number',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 455,
            'size' => 9
        ],
        [
            'name' => 'person_address_rest',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 464,
            'size' => 25
        ],
        [
            'name' => 'person_phone_ddd',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 489,
            'size' => 4
        ],
        [
            'name' => 'person_phone_number',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 493,
            'size' => 9
        ],
        [
            'name' => 'contract_start_date',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 502,
            'size' => 8
        ],
        [
            'name' => 'contract_value',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 510,
            'size' => 15
        ],
        [
            'name' => 'complementary_data',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 525,
            'size' => 1
        ],
        [
            'name' => 'blank_2',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 526,
            'size' => 5
        ],
        [
            'name' => 'communication_type',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 531,
            'size' => 1
        ],
        [
            'name' => 'blank_3',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 532,
            'size' => 2
        ],
        [
            'name' => 'errors_codes',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 534,
            'size' => 60
        ],
        [
            'name' => 'sequence',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 594,
            'size' => 7
        ],
    ];


    /**
     * @param string $cnpj_branch_digit Digito da filial no CNPJ da impresa
     * @param array $values Valores das colunas do detalhe
     * @param int | null  $sequence Sequência do detalhe no CNAB
     */
    public function __construct($cnpj_branch_digit, array $values, $sequence = null) {
        $this->createColumns();
        $this->setSequence($sequence);
        $this->setCnpjBranchDigit($cnpj_branch_digit);
        $this->validate($values);
        $this->setValues($values);
    }

    /**
     * Define o valor da sequência
     * @param int | null  $sequence Sequência do detalhe no CNAB
     */
    public function setSequence($sequence) {
        $this->sequence = $sequence;
    }

    /**
     * Define o valor do digito da filial
     * @param string $cnpj_branch_digit Digito da filial no CNPJ da impresa
     */
    public function setCnpjBranchDigit($cnpj_branch_digit) {
        $this->cnpj_branch_digit = $cnpj_branch_digit;
    }

    /**
     * @param array $values Define os valores das colunas 
     */
    private function setValues($values) {
        
        $values['occurrence_date'] = Util::dateToInt($values['occurrence_date']);
        $values['contract_end_date'] = Util::dateToInt($values['contract_end_date']);
        $values['person_birthdate'] = Util::dateToInt($values['person_birthdate']);
        $values['person_full_address'] = Util::onlyLettersNumbersAndSpaces($values['person_full_address']);
        $values['contract_number'] = Util::onlyNumbers($values['contract_number']);
        $values['person_phone_ddd'] = Util::onlyNumbers($values['person_phone_ddd']);
        $values['person_phone_number'] = Util::onlyNumbers($values['person_phone_number']);
        $values['contract_start_date'] = Util::dateToInt($values['contract_start_date']);
        
        $values['code'] = $this->code;
        $values['sequence'] = $this->sequence;
        $values['cnpj_branch_digit'] = $this->cnpj_branch_digit;
        

        if(empty($values['guarantor_first_document'])) {
            $values['guarantor_first_document'] = '               '; //Provisório (até...?), pois o if anterior não mudava o tipo de alfanumérico para numérico
        }       

        $this->setColumnsValues($values);
        $this->getColumn('person_address_rest')->setValue($this->getColumn('person_full_address')->getRest());
    }

    /**
     * Valida os valores
     * @param array $values
     */
    private function validate($values) {
        $this->validateOperation($values['operation']);
        // Fazer mais validações
    }

    /**
     * Valida o tipo de operação
     * @param string $operation
     */
    private function validateOperation($operation) {
        if($operation != 'I' && $operation != 'E') {
            throw new \Exception('Informed operation does not exists');
        }
    }
}